@@ -23,6 +23,7 @@
 #endif
 
 #include "gedit-app.h"
+#include "gedit-app-private.h"
 
 #include <string.h>
 #include <unistd.h>
@@ -76,7 +77,7 @@ typedef struct
 	GtkPageSetup      *page_setup;
 	GtkPrintSettings  *print_settings;
 
-	GObject           *settings;
+	GeditSettings     *settings;
 	GSettings         *ui_settings;
 	GSettings         *window_settings;
 
@@ -781,7 +782,7 @@ gedit_app_startup (GApplication *application)
 	priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
 
 	/* initial lockdown state */
-	priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (priv->settings));
+	priv->lockdown = gedit_settings_get_lockdown (priv->settings);
 
 	g_action_map_add_action_entries (G_ACTION_MAP (application),
 	                                 app_entries,
@@ -1860,7 +1861,7 @@ _gedit_app_set_default_print_settings (GeditApp         *app,
 	priv->print_settings = g_object_ref (settings);
 }
 
-GObject *
+GeditSettings *
 _gedit_app_get_settings (GeditApp *app)
 {
 	GeditAppPrivate *priv;
