@@ -1003,6 +1003,7 @@ gedit_commands_save_document_async (GeditDocument       *document,
 {
 	GTask *task;
 	GeditTab *tab;
+	GtkSourceFile *file;
 	gchar *uri_for_display;
 
 	gedit_debug (DEBUG_COMMANDS);
@@ -1014,9 +1015,10 @@ gedit_commands_save_document_async (GeditDocument       *document,
 	task = g_task_new (document, cancellable, callback, user_data);
 
 	tab = gedit_tab_get_from_document (document);
+	file = gedit_document_get_file (document);
 
 	if (gedit_document_is_untitled (document) ||
-	    gedit_document_get_readonly (document))
+	    gtk_source_file_is_readonly (file))
 	{
 		gedit_debug_message (DEBUG_COMMANDS, "Untitled or Readonly");
 
@@ -1299,19 +1301,19 @@ save_documents_list (GeditWindow *window,
 		state = gedit_tab_get_state (tab);
 
 		g_return_if_fail (state != GEDIT_TAB_STATE_PRINTING);
-		g_return_if_fail (state != GEDIT_TAB_STATE_PRINT_PREVIEWING);
 		g_return_if_fail (state != GEDIT_TAB_STATE_CLOSING);
 
 		if (state == GEDIT_TAB_STATE_NORMAL ||
-		    state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW ||
-		    state == GEDIT_TAB_STATE_GENERIC_NOT_EDITABLE)
+		    state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
 		{
 			if (_gedit_document_needs_saving (doc))
 			{
+				GtkSourceFile *file = gedit_document_get_file (doc);
+
 				/* FIXME: manage the case of local readonly files owned by the
 				   user is running gedit - Paolo (Dec. 8, 2005) */
 				if (gedit_document_is_untitled (doc) ||
-				    gedit_document_get_readonly (doc))
+				    gtk_source_file_is_readonly (file))
 				{
 					if (data == NULL)
 					{
@@ -1337,7 +1339,7 @@ save_documents_list (GeditWindow *window,
 			   - GEDIT_TAB_STATE_REVERTING: we do not save since the user wants
 			     to return back to the version of the file she previously saved
 			   - GEDIT_TAB_STATE_SAVING: well, we are already saving (no need to save again)
-			   - GEDIT_TAB_STATE_PRINTING, GEDIT_TAB_STATE_PRINT_PREVIEWING: there is not a
+			   - GEDIT_TAB_STATE_PRINTING: there is not a
 			     real reason for not saving in this case, we do not save to avoid to run
 			     two operations using the message area at the same time (may be we can remove
 			     this limitation in the future). Note that SaveAll, ClosAll
@@ -1719,7 +1721,7 @@ save_and_close_documents (GList         *docs,
 		     the original file has been deleted)
 		   - [*] GEDIT_TAB_STATE_SAVING: invalid, ClosAll
 		     and Quit are unsensitive if the window state is SAVING.
-		   - [*] GEDIT_TAB_STATE_PRINTING, GEDIT_TAB_STATE_PRINT_PREVIEWING: there is not a
+		   - [*] GEDIT_TAB_STATE_PRINTING: there is not a
 		     real reason for not closing in this case, we do not save to avoid to run
 		     two operations using the message area at the same time (may be we can remove
 		     this limitation in the future). Note that ClosAll
@@ -1735,7 +1737,6 @@ save_and_close_documents (GList         *docs,
 		*/
 
 		g_return_if_fail (state != GEDIT_TAB_STATE_PRINTING);
-		g_return_if_fail (state != GEDIT_TAB_STATE_PRINT_PREVIEWING);
 		g_return_if_fail (state != GEDIT_TAB_STATE_CLOSING);
 		g_return_if_fail (state != GEDIT_TAB_STATE_SAVING);
 
@@ -1748,13 +1749,15 @@ save_and_close_documents (GList         *docs,
 			    state != GEDIT_TAB_STATE_LOADING_ERROR &&
 			    state != GEDIT_TAB_STATE_REVERTING) /* FIXME: is this the right behavior with REVERTING ?*/
 			{
+				GtkSourceFile *file = gedit_document_get_file (doc);
+
 				/* The document must be saved before closing */
 				g_return_if_fail (_gedit_document_needs_saving (doc));
 
 				/* FIXME: manage the case of local readonly files owned by the
 				 * user is running gedit - Paolo (Dec. 8, 2005) */
 				if (gedit_document_is_untitled (doc) ||
-				    gedit_document_get_readonly (doc))
+				    gtk_source_file_is_readonly (file))
 				{
 					if (data == NULL)
 					{
