@@ -5,7 +5,7 @@
  * Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
  * Copyright (C) 2000, 2001 Chema Celorio, Paolo Maggi
  * Copyright (C) 2002-2005 Paolo Maggi
- * Copyright (C) 2014 Sébastien Wilmet
+ * Copyright (C) 2014-2015 Sébastien Wilmet
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,12 +34,9 @@
 #include "gedit-settings.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-
-#ifndef ENABLE_GVFS_METADATA
 #include "gedit-metadata-manager.h"
-#else
+
 #define METADATA_QUERY "metadata::*"
-#endif
 
 #define NO_LANGUAGE_NAME "_NORMAL_"
 
@@ -47,6 +44,9 @@ static void	gedit_document_loaded_real	(GeditDocument *doc);
 
 static void	gedit_document_saved_real	(GeditDocument *doc);
 
+static void	set_content_type		(GeditDocument *doc,
+						 const gchar   *content_type);
+
 typedef struct
 {
 	GtkSourceFile *file;
@@ -60,7 +60,6 @@ typedef struct
 
 	gchar	    *content_type;
 
-	GTimeVal     mtime;
 	GTimeVal     time_of_last_save_or_load;
 
 	/* The search context for the incremental search, or the search and
@@ -70,11 +69,8 @@ typedef struct
 
 	guint user_action;
 
-	guint readonly : 1;
-	guint externally_modified : 1;
-	guint deleted : 1;
 	guint language_set_by_user : 1;
-	guint mtime_set : 1;
+	guint use_gvfs_metadata : 1;
 
 	/* The search is empty if there is no search context, or if the
 	 * search text is empty. It is used for the sensitivity of some menu
@@ -95,7 +91,8 @@ enum
 	PROP_CONTENT_TYPE,
 	PROP_MIME_TYPE,
 	PROP_READ_ONLY,
-	PROP_EMPTY_SEARCH
+	PROP_EMPTY_SEARCH,
+	PROP_USE_GVFS_METADATA
 };
 
 enum
@@ -265,13 +262,17 @@ gedit_document_get_property (GObject    *object,
 			break;
 
 		case PROP_READ_ONLY:
-			g_value_set_boolean (value, priv->readonly);
+			g_value_set_boolean (value, gtk_source_file_is_readonly (priv->file));
 			break;
 
 		case PROP_EMPTY_SEARCH:
 			g_value_set_boolean (value, priv->empty_search);
 			break;
 
+		case PROP_USE_GVFS_METADATA:
+			g_value_set_boolean (value, priv->use_gvfs_metadata);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -285,15 +286,22 @@ gedit_document_set_property (GObject      *object,
 			     GParamSpec   *pspec)
 {
 	GeditDocument *doc = GEDIT_DOCUMENT (object);
+	GeditDocumentPrivate *priv = gedit_document_get_instance_private (doc);
 
 	switch (prop_id)
 	{
 		case PROP_SHORTNAME:
+			G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 			gedit_document_set_short_name_for_display (doc, g_value_get_string (value));
+			G_GNUC_END_IGNORE_DEPRECATIONS;
 			break;
 
 		case PROP_CONTENT_TYPE:
-			gedit_document_set_content_type (doc, g_value_get_string (value));
+			set_content_type (doc, g_value_get_string (value));
+			break;
+
+		case PROP_USE_GVFS_METADATA:
+			priv->use_gvfs_metadata = g_value_get_boolean (value);
 			break;
 
 		default:
@@ -423,12 +431,20 @@ gedit_document_class_init (GeditDocumentClass *klass)
 							      G_PARAM_READABLE |
 							      G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GeditDocument:read-only:
+	 *
+	 * Whether the document is read-only or not.
+	 *
+	 * Deprecated: 3.18: Use the #GtkSourceFile API.
+	 */
 	g_object_class_install_property (object_class, PROP_READ_ONLY,
 					 g_param_spec_boolean ("read-only",
 							       "Read Only",
-							       "Whether the document is read only or not",
+							       "Whether the document is read-only or not",
 							       FALSE,
 							       G_PARAM_READABLE |
+							       G_PARAM_DEPRECATED |
 							       G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -447,6 +463,27 @@ gedit_document_class_init (GeditDocumentClass *klass)
 							       G_PARAM_READABLE |
 							       G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GeditDocument:use-gvfs-metadata:
+	 *
+	 * Whether to use GVFS metadata. If %FALSE, use the gedit metadata
+	 * manager that stores the metadata in an XML file in the user cache
+	 * directory.
+	 *
+	 * <warning>
+	 * The property is used internally by gedit. It must not be used in a
+	 * gedit plugin. The property can be modified or removed at any time.
+	 * </warning>
+	 */
+	g_object_class_install_property (object_class, PROP_USE_GVFS_METADATA,
+					 g_param_spec_boolean ("use-gvfs-metadata",
+							       "Use GVFS metadata",
+							       "",
+							       TRUE,
+							       G_PARAM_READWRITE |
+							       G_PARAM_CONSTRUCT_ONLY |
+							       G_PARAM_STATIC_STRINGS));
+
 	/* This signal is used to update the cursor position in the statusbar,
 	 * it's emitted either when the insert mark is moved explicitely or
 	 * when the buffer changes (insert/delete).
@@ -565,22 +602,6 @@ set_language (GeditDocument     *doc,
 
 	gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (doc), lang);
 
-	if (lang != NULL)
-	{
-		gboolean syntax_hl;
-
-		syntax_hl = g_settings_get_boolean (priv->editor_settings,
-						    GEDIT_SETTINGS_SYNTAX_HIGHLIGHTING);
-
-		gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (doc),
-							syntax_hl);
-	}
-	else
-	{
-		gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (doc),
-							FALSE);
-	}
-
 	if (set_by_user)
 	{
 		const gchar *language = get_language_string (doc);
@@ -747,13 +768,11 @@ on_location_changed (GtkSourceFile *file,
 		g_object_notify (G_OBJECT (doc), "shortname");
 	}
 
-#ifdef ENABLE_GVFS_METADATA
-
-	/* load metadata for this location: we load sync since metadata is
+	/* Load metadata for this location: we load sync since metadata is
 	 * always local so it should be fast and we need the information
 	 * right after the location was set.
 	 */
-	if (location != NULL)
+	if (priv->use_gvfs_metadata && location != NULL)
 	{
 		GError *error = NULL;
 
@@ -784,7 +803,14 @@ on_location_changed (GtkSourceFile *file,
 			g_error_free (error);
 		}
 	}
-#endif
+}
+
+static void
+on_readonly_changed (GtkSourceFile *file,
+		     GParamSpec    *pspec,
+		     GeditDocument *doc)
+{
+	g_object_notify (G_OBJECT (doc), "read-only");
 }
 
 static void
@@ -800,7 +826,6 @@ gedit_document_init (GeditDocument *doc)
 	priv->editor_settings = g_settings_new ("org.gnome.gedit.preferences.editor");
 	priv->untitled_number = get_untitled_number ();
 	priv->content_type = get_default_content_type ();
-	priv->readonly = FALSE;
 	priv->language_set_by_user = FALSE;
 	priv->empty_search = TRUE;
 
@@ -814,17 +839,29 @@ gedit_document_init (GeditDocument *doc)
 				 doc,
 				 0);
 
+	g_signal_connect_object (priv->file,
+				 "notify::read-only",
+				 G_CALLBACK (on_readonly_changed),
+				 doc,
+				 0);
+
 	g_settings_bind (priv->editor_settings,
 	                 GEDIT_SETTINGS_MAX_UNDO_ACTIONS,
 	                 doc,
 	                 "max-undo-levels",
-	                 G_SETTINGS_BIND_GET);
+	                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_NO_SENSITIVITY);
+
+	g_settings_bind (priv->editor_settings,
+			 GEDIT_SETTINGS_SYNTAX_HIGHLIGHTING,
+			 doc,
+			 "highlight-syntax",
+			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_NO_SENSITIVITY);
 
 	g_settings_bind (priv->editor_settings,
 	                 GEDIT_SETTINGS_BRACKET_MATCHING,
 	                 doc,
 	                 "highlight-matching-brackets",
-	                 G_SETTINGS_BIND_GET);
+	                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_NO_SENSITIVITY);
 
 	style_scheme = get_default_style_scheme (priv->editor_settings);
 	if (style_scheme != NULL)
@@ -841,7 +878,17 @@ gedit_document_init (GeditDocument *doc)
 GeditDocument *
 gedit_document_new (void)
 {
-	return g_object_new (GEDIT_TYPE_DOCUMENT, NULL);
+	gboolean use_gvfs_metadata;
+
+#ifdef ENABLE_GVFS_METADATA
+	use_gvfs_metadata = TRUE;
+#else
+	use_gvfs_metadata = FALSE;
+#endif
+
+	return g_object_new (GEDIT_TYPE_DOCUMENT,
+			     "use-gvfs-metadata", use_gvfs_metadata,
+			     NULL);
 }
 
 static gchar *
@@ -916,19 +963,12 @@ set_content_type_no_guess (GeditDocument *doc,
 	g_object_notify (G_OBJECT (doc), "content-type");
 }
 
-/**
- * gedit_document_set_content_type:
- * @doc:
- * @content_type: (allow-none):
- */
-void
-gedit_document_set_content_type (GeditDocument *doc,
-                                 const gchar   *content_type)
+static void
+set_content_type (GeditDocument *doc,
+		  const gchar   *content_type)
 {
 	GeditDocumentPrivate *priv;
 
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-
 	gedit_debug (DEBUG_DOCUMENT);
 
 	priv = gedit_document_get_instance_private (doc);
@@ -959,6 +999,23 @@ gedit_document_set_content_type (GeditDocument *doc,
 	}
 }
 
+/**
+ * gedit_document_set_content_type:
+ * @doc:
+ * @content_type: (allow-none):
+ *
+ * Deprecated: 3.18: Unused function. The intent is to change the
+ * #GeditDocument:content-type property to be read-only.
+ */
+void
+gedit_document_set_content_type (GeditDocument *doc,
+                                 const gchar   *content_type)
+{
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	set_content_type (doc, content_type);
+}
+
 /**
  * gedit_document_get_location:
  * @doc: a #GeditDocument
@@ -1003,7 +1060,7 @@ gedit_document_set_location (GeditDocument *doc,
 	priv = gedit_document_get_instance_private (doc);
 
 	gtk_source_file_set_location (priv->file, location);
-	gedit_document_set_content_type (doc, NULL);
+	set_content_type (doc, NULL);
 }
 
 /**
@@ -1072,6 +1129,9 @@ gedit_document_get_short_name_for_display (GeditDocument *doc)
  * gedit_document_set_short_name_for_display:
  * @doc:
  * @short_name: (allow-none):
+ *
+ * Deprecated: 3.18: Unused function. The intent is to change the
+ * #GeditDocument:shortname property to be read-only.
  */
 void
 gedit_document_set_short_name_for_display (GeditDocument *doc,
@@ -1125,27 +1185,13 @@ gedit_document_get_mime_type (GeditDocument *doc)
 	return g_strdup ("text/plain");
 }
 
-static void
-set_readonly (GeditDocument *doc,
-	      gboolean       readonly)
-{
-	GeditDocumentPrivate *priv;
-
-	gedit_debug (DEBUG_DOCUMENT);
-
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-
-	priv = gedit_document_get_instance_private (doc);
-
-	readonly = readonly != FALSE;
-
-	if (priv->readonly != readonly)
-	{
-		priv->readonly = readonly;
-		g_object_notify (G_OBJECT (doc), "read-only");
-	}
-}
-
+/**
+ * gedit_document_get_readonly:
+ * @doc: a #GeditDocument.
+ *
+ * Returns: whether the document is read-only.
+ * Deprecated: 3.18: Use gtk_source_file_is_readonly() instead.
+ */
 gboolean
 gedit_document_get_readonly (GeditDocument *doc)
 {
@@ -1155,7 +1201,7 @@ gedit_document_get_readonly (GeditDocument *doc)
 
 	priv = gedit_document_get_instance_private (doc);
 
-	return priv->readonly;
+	return gtk_source_file_is_readonly (priv->file);
 }
 
 static void
@@ -1163,12 +1209,9 @@ loaded_query_info_cb (GFile         *location,
 		      GAsyncResult  *result,
 		      GeditDocument *doc)
 {
-	GeditDocumentPrivate *priv;
 	GFileInfo *info;
 	GError *error = NULL;
 
-	priv = gedit_document_get_instance_private (doc);
-
 	info = g_file_query_info_finish (location, result, &error);
 
 	if (error != NULL)
@@ -1186,35 +1229,18 @@ loaded_query_info_cb (GFile         *location,
 		error = NULL;
 	}
 
-	if (info != NULL)
+	if (info != NULL &&
+	    g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE))
 	{
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE))
-		{
-			const gchar *content_type;
+		const gchar *content_type;
 
-			content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+		content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
 
-			gedit_document_set_content_type (doc, content_type);
-		}
-
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
-		{
-			gboolean read_only;
-
-			read_only = !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
-
-			set_readonly (doc, read_only);
-		}
-
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
-		{
-			g_file_info_get_modification_time (info, &priv->mtime);
-			priv->mtime_set = TRUE;
-		}
-
-		g_object_unref (info);
+		set_content_type (doc, content_type);
 	}
 
+	g_clear_object (&info);
+
 	/* Async operation finished. */
 	g_object_unref (doc);
 }
@@ -1237,15 +1263,9 @@ gedit_document_loaded_real (GeditDocument *doc)
 		set_language (doc, language, FALSE);
 	}
 
-	priv->mtime_set = FALSE;
-	priv->externally_modified = FALSE;
-	priv->deleted = FALSE;
-
 	g_get_current_time (&priv->time_of_last_save_or_load);
 
-	set_readonly (doc, FALSE);
-
-	gedit_document_set_content_type (doc, NULL);
+	set_content_type (doc, NULL);
 
 	location = gtk_source_file_get_location (priv->file);
 
@@ -1256,8 +1276,7 @@ gedit_document_loaded_real (GeditDocument *doc)
 
 		g_file_query_info_async (location,
 					 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-					 G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE ","
-					 G_FILE_ATTRIBUTE_TIME_MODIFIED,
+					 G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
 					 G_FILE_QUERY_INFO_NONE,
 					 G_PRIORITY_DEFAULT,
 					 NULL,
@@ -1287,23 +1306,13 @@ saved_query_info_cb (GFile         *location,
 		error = NULL;
 	}
 
-	priv->mtime_set = FALSE;
-
-	if (info != NULL)
+	if (info != NULL &&
+	    g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE))
 	{
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE))
-		{
-			content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
-		}
-
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
-		{
-			g_file_info_get_modification_time (info, &priv->mtime);
-			priv->mtime_set = TRUE;
-		}
+		content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
 	}
 
-	gedit_document_set_content_type (doc, content_type);
+	set_content_type (doc, content_type);
 
 	if (info != NULL)
 	{
@@ -1313,12 +1322,8 @@ saved_query_info_cb (GFile         *location,
 
 	g_get_current_time (&priv->time_of_last_save_or_load);
 
-	priv->externally_modified = FALSE;
-	priv->deleted = FALSE;
 	priv->create = FALSE;
 
-	set_readonly (doc, FALSE);
-
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);
 
 	save_encoding_metadata (doc);
@@ -1341,8 +1346,7 @@ gedit_document_saved_real (GeditDocument *doc)
 	g_object_ref (doc);
 
 	g_file_query_info_async (location,
-				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
-				 G_FILE_ATTRIBUTE_TIME_MODIFIED,
+				 G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 				 G_FILE_QUERY_INFO_NONE,
 				 G_PRIORITY_DEFAULT,
 				 NULL,
@@ -1377,101 +1381,33 @@ gedit_document_is_untitled (GeditDocument *doc)
 	return gtk_source_file_get_location (priv->file) == NULL;
 }
 
+/**
+ * gedit_document_is_local:
+ * @doc: a #GeditDocument.
+ *
+ * Returns: whether the document is local.
+ * Deprecated: 3.18: Use gtk_source_file_is_local() instead.
+ */
 gboolean
 gedit_document_is_local (GeditDocument *doc)
 {
 	GeditDocumentPrivate *priv;
-	GFile *location;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 
 	priv = gedit_document_get_instance_private (doc);
 
-	location = gtk_source_file_get_location (priv->file);
-
-	if (location == NULL)
-	{
-		return FALSE;
-	}
-
-	return g_file_has_uri_scheme (location, "file");
-}
-
-static void
-check_file_on_disk (GeditDocument *doc)
-{
-	GeditDocumentPrivate *priv;
-	GFile *location;
-	GFileInfo *info;
-
-	priv = gedit_document_get_instance_private (doc);
-
-	location = gtk_source_file_get_location (priv->file);
-
-	if (location == NULL)
-	{
-		return;
-	}
-
-	info = g_file_query_info (location,
-				  G_FILE_ATTRIBUTE_TIME_MODIFIED "," \
-				  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
-				  G_FILE_QUERY_INFO_NONE,
-				  NULL, NULL);
-
-	if (info != NULL)
-	{
-		/* While at it also check if permissions changed */
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
-		{
-			gboolean read_only;
-
-			read_only = !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
-
-			set_readonly (doc, read_only);
-		}
-
-		if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED) && priv->mtime_set)
-		{
-			GTimeVal timeval;
-
-			g_file_info_get_modification_time (info, &timeval);
-
-			/* Note that mtime can even go backwards if the
-			 * user is copying over a file with an old mtime
-			 */
-			if (timeval.tv_sec != priv->mtime.tv_sec ||
-			    timeval.tv_usec != priv->mtime.tv_usec)
-			{
-				priv->externally_modified = TRUE;
-			}
-		}
-
-		g_object_unref (info);
-	}
-	else
-	{
-		priv->deleted = TRUE;
-	}
-}
-
-gboolean
-_gedit_document_check_externally_modified (GeditDocument *doc)
-{
-	GeditDocumentPrivate *priv;
-
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
-
-	priv = gedit_document_get_instance_private (doc);
-
-	if (!priv->externally_modified)
-	{
-		check_file_on_disk (doc);
-	}
-
-	return priv->externally_modified;
+	return gtk_source_file_is_local (priv->file);
 }
 
+/**
+ * gedit_document_get_deleted:
+ * @doc: a #GeditDocument.
+ *
+ * Returns: whether the file has been deleted.
+ *
+ * Deprecated: 3.18: Unused function.
+ */
 gboolean
 gedit_document_get_deleted (GeditDocument *doc)
 {
@@ -1481,12 +1417,7 @@ gedit_document_get_deleted (GeditDocument *doc)
 
 	priv = gedit_document_get_instance_private (doc);
 
-	if (!priv->deleted)
-	{
-		check_file_on_disk (doc);
-	}
-
-	return priv->deleted;
+	return gtk_source_file_is_deleted (priv->file);
 }
 
 /*
@@ -1496,6 +1427,8 @@ gboolean
 _gedit_document_needs_saving (GeditDocument *doc)
 {
 	GeditDocumentPrivate *priv;
+	gboolean externally_modified = FALSE;
+	gboolean deleted = FALSE;
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 
@@ -1506,12 +1439,14 @@ _gedit_document_needs_saving (GeditDocument *doc)
 		return TRUE;
 	}
 
-	if (gedit_document_is_local (doc))
+	if (gtk_source_file_is_local (priv->file))
 	{
-		check_file_on_disk (doc);
+		gtk_source_file_check_file_on_disk (priv->file);
+		externally_modified = gtk_source_file_is_externally_modified (priv->file);
+		deleted = gtk_source_file_is_deleted (priv->file);
 	}
 
-	return (priv->externally_modified || priv->deleted) && !priv->create;
+	return (externally_modified || deleted) && !priv->create;
 }
 
 /* If @line is bigger than the lines of the document, the cursor is moved
@@ -1688,17 +1623,13 @@ gedit_document_get_compression_type (GeditDocument *doc)
 	return gtk_source_file_get_compression_type (priv->file);
 }
 
-#ifndef ENABLE_GVFS_METADATA
-gchar *
-gedit_document_get_metadata (GeditDocument *doc,
-			     const gchar   *key)
+static gchar *
+get_metadata_from_metadata_manager (GeditDocument *doc,
+				    const gchar   *key)
 {
 	GeditDocumentPrivate *priv;
 	GFile *location;
 
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
-	g_return_val_if_fail (key != NULL, NULL);
-
 	priv = gedit_document_get_instance_private (doc);
 
 	location = gtk_source_file_get_location (priv->file);
@@ -1711,42 +1642,59 @@ gedit_document_get_metadata (GeditDocument *doc,
 	return NULL;
 }
 
-void
-gedit_document_set_metadata (GeditDocument *doc,
-			     const gchar   *first_key,
-			     ...)
+static gchar *
+get_metadata_from_gvfs (GeditDocument *doc,
+			const gchar   *key)
 {
 	GeditDocumentPrivate *priv;
-	GFile *location;
-	const gchar *key;
-	const gchar *value;
-	va_list var_args;
-
-	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-	g_return_if_fail (first_key != NULL);
 
 	priv = gedit_document_get_instance_private (doc);
 
-	location = gtk_source_file_get_location (priv->file);
-
-	if (location == NULL)
+	if (priv->metadata_info != NULL &&
+	    g_file_info_has_attribute (priv->metadata_info, key))
 	{
-		/* Can't set metadata for untitled documents */
-		return;
+		return g_strdup (g_file_info_get_attribute_string (priv->metadata_info, key));
 	}
 
-	va_start (var_args, first_key);
-
-	for (key = first_key; key; key = va_arg (var_args, const gchar *))
-	{
-		value = va_arg (var_args, const gchar *);
-		gedit_metadata_manager_set (location, key, value);
-	}
-
-	va_end (var_args);
+	return NULL;
 }
 
-#else
+static void
+set_gvfs_metadata (GeditDocument *doc,
+		   GFileInfo     *info,
+		   const gchar   *key,
+		   const gchar   *value)
+{
+	GeditDocumentPrivate *priv;
+
+	priv = gedit_document_get_instance_private (doc);
+
+	if (value != NULL)
+	{
+		g_file_info_set_attribute_string (info, key, value);
+
+		if (priv->metadata_info != NULL)
+		{
+			g_file_info_set_attribute_string (priv->metadata_info, key, value);
+		}
+	}
+	else
+	{
+		/* Unset the key */
+		g_file_info_set_attribute (info,
+					   key,
+					   G_FILE_ATTRIBUTE_TYPE_INVALID,
+					   NULL);
+
+		if (priv->metadata_info != NULL)
+		{
+			g_file_info_set_attribute (priv->metadata_info,
+						   key,
+						   G_FILE_ATTRIBUTE_TYPE_INVALID,
+						   NULL);
+		}
+	}
+}
 
 /**
  * gedit_document_get_metadata:
@@ -1768,13 +1716,12 @@ gedit_document_get_metadata (GeditDocument *doc,
 
 	priv = gedit_document_get_instance_private (doc);
 
-	if (priv->metadata_info != NULL &&
-	    g_file_info_has_attribute (priv->metadata_info, key))
+	if (priv->use_gvfs_metadata)
 	{
-		return g_strdup (g_file_info_get_attribute_string (priv->metadata_info, key));
+		return get_metadata_from_gvfs (doc, key);
 	}
 
-	return NULL;
+	return get_metadata_from_metadata_manager (doc, key);
 }
 
 /**
@@ -1792,51 +1739,53 @@ gedit_document_set_metadata (GeditDocument *doc,
 			     ...)
 {
 	GeditDocumentPrivate *priv;
-	const gchar *key;
-	const gchar *value;
-	va_list var_args;
-	GFileInfo *info;
 	GFile *location;
+	const gchar *key;
+	va_list var_args;
+	GFileInfo *info = NULL;
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (first_key != NULL);
 
 	priv = gedit_document_get_instance_private (doc);
 
-	info = g_file_info_new ();
+	location = gtk_source_file_get_location (priv->file);
+
+	/* With the metadata manager, can't set metadata for untitled documents.
+	 * With GVFS metadata, if the location is NULL the metadata is stored in
+	 * priv->metadata_info, so that it can be saved later if the document is
+	 * saved.
+	 */
+	if (!priv->use_gvfs_metadata && location == NULL)
+	{
+		return;
+	}
+
+	if (priv->use_gvfs_metadata)
+	{
+		info = g_file_info_new ();
+	}
 
 	va_start (var_args, first_key);
 
 	for (key = first_key; key; key = va_arg (var_args, const gchar *))
 	{
-		value = va_arg (var_args, const gchar *);
+		const gchar *value = va_arg (var_args, const gchar *);
 
-		if (value != NULL)
+		if (priv->use_gvfs_metadata)
 		{
-			g_file_info_set_attribute_string (info, key, value);
-
-			if (priv->metadata_info != NULL)
-			{
-				g_file_info_set_attribute_string (priv->metadata_info, key, value);
-			}
+			/* Collect the metadata into @info. */
+			set_gvfs_metadata (doc, info, key, value);
 		}
 		else
 		{
-			/* Unset the key */
-			g_file_info_remove_attribute (info, key);
-
-			if (priv->metadata_info != NULL)
-			{
-				g_file_info_remove_attribute (priv->metadata_info, key);
-			}
+			gedit_metadata_manager_set (location, key, value);
 		}
 	}
 
 	va_end (var_args);
 
-	location = gtk_source_file_get_location (priv->file);
-
-	if (location != NULL)
+	if (priv->use_gvfs_metadata && location != NULL)
 	{
 		GError *error = NULL;
 
@@ -1867,9 +1816,8 @@ gedit_document_set_metadata (GeditDocument *doc,
 		}
 	}
 
-	g_object_unref (info);
+	g_clear_object (&info);
 }
-#endif
 
 static void
 update_empty_search (GeditDocument *doc)
@@ -1964,7 +1912,7 @@ gedit_document_set_search_context (GeditDocument          *doc,
 		g_settings_bind (priv->editor_settings,
 				 GEDIT_SETTINGS_SEARCH_HIGHLIGHTING,
 				 search_context, "highlight",
-				 G_SETTINGS_BIND_GET);
+				 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_NO_SENSITIVITY);
 
 		g_signal_connect_object (search_context,
 					 "notify::settings",
