@@ -41,6 +41,8 @@
 #include "gedit-settings.h"
 #include "gedit-utils.h"
 #include "gedit-file-chooser-dialog.h"
+#include "gedit-app.h"
+#include "gedit-app-private.h"
 
 /*
  * gedit-preferences dialog is a singleton since we don't
@@ -436,7 +438,7 @@ setup_view_page (GeditPreferencesDialog *dlg)
 static void
 setup_font_colors_page_font_section (GeditPreferencesDialog *dlg)
 {
-	GObject *settings;
+	GeditSettings *settings;
 	gchar *system_font = NULL;
 	gchar *label;
 
@@ -447,7 +449,7 @@ setup_font_colors_page_font_section (GeditPreferencesDialog *dlg)
 
 	/* Get values */
 	settings = _gedit_app_get_settings (GEDIT_APP (g_application_get_default ()));
-	system_font = gedit_settings_get_system_font (GEDIT_SETTINGS (settings));
+	system_font = gedit_settings_get_system_font (settings);
 
 	label = g_strdup_printf(_("_Use the system fixed width font (%s)"),
 				system_font);
