@@ -21,10 +21,6 @@
  * Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gedit-view-frame.h"
 
 #include <gtksourceview/gtksource.h>
@@ -32,7 +28,6 @@
 #include <glib/gi18n.h>
 #include <stdlib.h>
 
-#include "gedit-window.h"
 #include "gedit-view-holder.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
@@ -63,7 +58,6 @@ struct _GeditViewFrame
 
 	GeditView *view;
 	GeditViewHolder *view_holder;
-	GtkWidget *window;
 	GtkFrame *map_frame;
 
 	SearchMode search_mode;
@@ -100,19 +94,16 @@ struct _GeditViewFrame
 	 */
 	gchar *search_text;
 	gchar *old_search_text;
-
-	gint window_state_changed_handler_id;
-};
-
-enum
-{
-	PROP_0,
-	PROP_DOCUMENT,
-	PROP_VIEW
 };
 
 G_DEFINE_TYPE (GeditViewFrame, gedit_view_frame, GTK_TYPE_OVERLAY)
 
+static GeditDocument *
+get_document (GeditViewFrame *frame)
+{
+	return GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (frame->view)));
+}
+
 static void
 gedit_view_frame_dispose (GObject *object)
 {
@@ -170,40 +161,9 @@ gedit_view_frame_finalize (GObject *object)
 	g_free (frame->search_text);
 	g_free (frame->old_search_text);
 
-	if (frame->window_state_changed_handler_id != 0)
-	{
-		g_signal_handler_disconnect (frame->window,
-		                             frame->window_state_changed_handler_id);
-		frame->window_state_changed_handler_id = 0;
-	}
-
 	G_OBJECT_CLASS (gedit_view_frame_parent_class)->finalize (object);
 }
 
-static void
-gedit_view_frame_get_property (GObject    *object,
-                               guint       prop_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
-{
-	GeditViewFrame *frame = GEDIT_VIEW_FRAME (object);
-
-	switch (prop_id)
-	{
-		case PROP_DOCUMENT:
-			g_value_set_object (value, gedit_view_frame_get_document (frame));
-			break;
-
-		case PROP_VIEW:
-			g_value_set_object (value, gedit_view_frame_get_view (frame));
-			break;
-
-		default:
-			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-			break;
-	}
-}
-
 static void
 hide_search_widget (GeditViewFrame *frame,
                     gboolean        cancel)
@@ -280,7 +240,7 @@ get_search_context (GeditViewFrame *frame)
 	GtkSourceSearchContext *search_context;
 	GtkSourceSearchSettings *search_settings;
 
-	doc = gedit_view_frame_get_document (frame);
+	doc = get_document (frame);
 	search_context = gedit_document_get_search_context (doc);
 
 	if (search_context == NULL)
@@ -1091,7 +1051,7 @@ update_goto_line (GeditViewFrame *frame)
 		return;
 	}
 
-	doc = gedit_view_frame_get_document (frame);
+	doc = get_document (frame);
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),
 					  &iter,
@@ -1452,23 +1412,6 @@ gedit_view_frame_class_init (GeditViewFrameClass *klass)
 
 	object_class->dispose = gedit_view_frame_dispose;
 	object_class->finalize = gedit_view_frame_finalize;
-	object_class->get_property = gedit_view_frame_get_property;
-
-	g_object_class_install_property (object_class, PROP_DOCUMENT,
-	                                 g_param_spec_object ("document",
-	                                                      "Document",
-	                                                      "The Document",
-	                                                      GEDIT_TYPE_DOCUMENT,
-	                                                      G_PARAM_READABLE |
-	                                                      G_PARAM_STATIC_STRINGS));
-
-	g_object_class_install_property (object_class, PROP_VIEW,
-	                                 g_param_spec_object ("view",
-	                                                      "View",
-	                                                      "The View",
-	                                                      GEDIT_TYPE_VIEW,
-	                                                      G_PARAM_READABLE |
-	                                                      G_PARAM_STATIC_STRINGS));
 
 	/* Bind class to template */
 	gtk_widget_class_set_template_from_resource (widget_class,
@@ -1492,47 +1435,6 @@ view_frame_mount_operation_factory (GtkSourceFile *file,
 	return gtk_mount_operation_new (GTK_WINDOW (window));
 }
 
-static gboolean
-on_window_state_changed (GtkWidget           *widget,
-                         GdkEventWindowState *event,
-                         GeditViewFrame      *frame)
-{
-	gboolean fullscreen_state;
-
-	fullscreen_state = event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
-
-	gedit_view_holder_set_centering (frame->view_holder, fullscreen_state);
-
-	return FALSE;
-}
-
-static void
-on_toplevel_window_changed (GtkWidget      *widget,
-                            GtkWidget      *previous_toplevel,
-                            GeditViewFrame *frame)
-{
-	GeditWindow *gedit_window;
-
-	if (frame->window_state_changed_handler_id != 0)
-	{
-		g_signal_handler_disconnect (frame->window,
-		                             frame->window_state_changed_handler_id);
-		frame->window_state_changed_handler_id = 0;
-	}
-
-	gedit_window = GEDIT_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (frame), GEDIT_TYPE_WINDOW));
-
-	if (gedit_window)
-	{
-		frame->window = GTK_WIDGET (&gedit_window->window);
-
-		frame->window_state_changed_handler_id = g_signal_connect (frame->window,
-		                                                          "window-state-event",
-		                                                          G_CALLBACK (on_window_state_changed),
-		                                                          frame);
-	}
-}
-
 static void
 gedit_view_frame_init (GeditViewFrame *frame)
 {
@@ -1553,11 +1455,9 @@ gedit_view_frame_init (GeditViewFrame *frame)
 
 	gtk_widget_override_background_color (GTK_WIDGET (frame), 0, &transparent);
 
-	doc = gedit_view_frame_get_document (frame);
+	doc = get_document (frame);
 	file = gedit_document_get_file (doc);
 
-	frame->window_state_changed_handler_id = 0;
-
 	gtk_source_file_set_mount_operation_factory (file,
 						     view_frame_mount_operation_factory,
 						     frame,
@@ -1644,11 +1544,6 @@ gedit_view_frame_init (GeditViewFrame *frame)
 				  "clicked",
 				  G_CALLBACK (forward_search),
 				  frame);
-
-	g_signal_connect (frame,
-	                  "hierarchy-changed",
-	                  G_CALLBACK (on_toplevel_window_changed),
-	                  frame);
 }
 
 GeditViewFrame *
@@ -1657,12 +1552,12 @@ gedit_view_frame_new (void)
 	return g_object_new (GEDIT_TYPE_VIEW_FRAME, NULL);
 }
 
-GeditDocument *
-gedit_view_frame_get_document (GeditViewFrame *frame)
+GeditViewHolder *
+gedit_view_frame_get_view_holder (GeditViewFrame *frame)
 {
 	g_return_val_if_fail (GEDIT_IS_VIEW_FRAME (frame), NULL);
 
-	return GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (frame->view)));
+	return frame->view_holder;
 }
 
 GeditView *
