@@ -20,15 +20,8 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gedit-view.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <gdk/gdkkeysyms.h>
 #include <libpeas/peas-extension-set.h>
 #include <glib/gi18n.h>
 
@@ -39,15 +32,14 @@
 #include "gedit-utils.h"
 #include "gedit-settings.h"
 #include "gedit-app.h"
-#include "gedit-notebook.h"
+#include "gedit-app-private.h"
 
 #define GEDIT_VIEW_SCROLL_MARGIN 0.02
 
 enum
 {
 	TARGET_URI_LIST = 100,
-	TARGET_XDNDDIRECTSAVE,
-	TARGET_TAB
+	TARGET_XDNDDIRECTSAVE
 };
 
 struct _GeditViewPrivate
@@ -60,14 +52,13 @@ struct _GeditViewPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE (GeditView, gedit_view, GTK_SOURCE_TYPE_VIEW)
 
-/* Signals */
 enum
 {
 	DROP_URIS,
 	LAST_SIGNAL
 };
 
-static guint view_signals [LAST_SIGNAL] = { 0 };
+static guint view_signals[LAST_SIGNAL] = { 0 };
 
 static void
 file_read_only_notify_handler (GtkSourceFile *file,
@@ -100,7 +91,7 @@ current_buffer_removed (GeditView *view)
 
 static void
 on_notify_buffer_cb (GeditView  *view,
-		     GParamSpec *arg1,
+		     GParamSpec *pspec,
 		     gpointer    userdata)
 {
 	GtkTextBuffer *buffer;
@@ -109,8 +100,10 @@ on_notify_buffer_cb (GeditView  *view,
 	current_buffer_removed (view);
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
-	if (buffer == NULL || !GEDIT_IS_DOCUMENT (buffer))
+	if (!GEDIT_IS_DOCUMENT (buffer))
+	{
 		return;
+	}
 
 	file = gedit_document_get_file (GEDIT_DOCUMENT (buffer));
 
@@ -128,7 +121,7 @@ on_notify_buffer_cb (GeditView  *view,
 static void
 gedit_view_init (GeditView *view)
 {
-	GtkTargetList *tl;
+	GtkTargetList *target_list;
 
 	gedit_debug (DEBUG_VIEW);
 
@@ -138,19 +131,15 @@ gedit_view_init (GeditView *view)
 
 	/* Drag and drop support */
 	view->priv->direct_save_uri = NULL;
-	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (view));
+	target_list = gtk_drag_dest_get_target_list (GTK_WIDGET (view));
 
-	if (tl != NULL)
+	if (target_list != NULL)
 	{
-		gtk_target_list_add (tl,
+		gtk_target_list_add (target_list,
 		                     gdk_atom_intern ("XdndDirectSave0", FALSE),
 		                     0,
 		                     TARGET_XDNDDIRECTSAVE);
-		gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
-		gtk_target_list_add (tl,
-		                     gdk_atom_intern_static_string ("GTK_NOTEBOOK_TAB"),
-		                     GTK_TARGET_SAME_APP,
-		                     TARGET_TAB);
+		gtk_target_list_add_uri_targets (target_list, TARGET_URI_LIST);
 	}
 
 	view->priv->extensions =
@@ -197,15 +186,14 @@ gedit_view_constructed (GObject *object)
 	view = GEDIT_VIEW (object);
 	priv = view->priv;
 
-	/* Get setting values */
 	use_default_font = g_settings_get_boolean (view->priv->editor_settings,
 	                                           GEDIT_SETTINGS_USE_DEFAULT_FONT);
 
-	/*
-	 *  Set tab, fonts, wrap mode, colors, etc. according
-	 *  to preferences
-	 */
-	if (!use_default_font)
+	if (use_default_font)
+	{
+		gedit_view_set_font (view, TRUE, NULL);
+	}
+	else
 	{
 		gchar *editor_font;
 
@@ -216,10 +204,6 @@ gedit_view_constructed (GObject *object)
 
 		g_free (editor_font);
 	}
-	else
-	{
-		gedit_view_set_font (view, TRUE, NULL);
-	}
 
 	g_settings_bind (priv->editor_settings,
 	                 GEDIT_SETTINGS_DISPLAY_LINE_NUMBERS,
@@ -281,21 +265,20 @@ gedit_view_constructed (GObject *object)
 	                 "smart-home-end",
 	                 G_SETTINGS_BIND_GET);
 
-	g_object_set (G_OBJECT (view),
-	              "indent_on_tab", TRUE,
-	              NULL);
+	gtk_source_view_set_indent_on_tab (GTK_SOURCE_VIEW (view), TRUE);
 
 	G_OBJECT_CLASS (gedit_view_parent_class)->constructed (object);
 }
 
-static gint
-gedit_view_focus_out (GtkWidget *widget, GdkEventFocus *event)
+static gboolean
+gedit_view_focus_out (GtkWidget     *widget,
+		      GdkEventFocus *event)
 {
 	gtk_widget_queue_draw (widget);
 
 	GTK_WIDGET_CLASS (gedit_view_parent_class)->focus_out_event (widget, event);
 
-	return FALSE;
+	return GDK_EVENT_PROPAGATE;
 }
 
 static GdkAtom
@@ -303,13 +286,13 @@ drag_get_uri_target (GtkWidget      *widget,
 		     GdkDragContext *context)
 {
 	GdkAtom target;
-	GtkTargetList *tl;
+	GtkTargetList *target_list;
 
-	tl = gtk_target_list_new (NULL, 0);
-	gtk_target_list_add_uri_targets (tl, 0);
+	target_list = gtk_target_list_new (NULL, 0);
+	gtk_target_list_add_uri_targets (target_list, 0);
 
-	target = gtk_drag_dest_find_target (widget, context, tl);
-	gtk_target_list_unref (tl);
+	target = gtk_drag_dest_find_target (widget, context, target_list);
+	gtk_target_list_unref (target_list);
 
 	return target;
 }
@@ -321,12 +304,16 @@ gedit_view_drag_motion (GtkWidget      *widget,
 			gint            y,
 			guint           timestamp)
 {
-	gboolean result;
+	gboolean drop_zone;
 
 	/* Chain up to allow textview to scroll and position dnd mark, note
 	 * that this needs to be checked if gtksourceview or gtktextview
-	 * changes drag_motion behaviour */
-	result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_motion (widget, context, x, y, timestamp);
+	 * changes drag_motion behaviour.
+	 */
+	drop_zone = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_motion (widget,
+									     context,
+									     x, y,
+									     timestamp);
 
 	/* If this is a URL, deal with it here */
 	if (drag_get_uri_target (widget, context) != GDK_NONE)
@@ -334,26 +321,10 @@ gedit_view_drag_motion (GtkWidget      *widget,
 		gdk_drag_status (context,
 				 gdk_drag_context_get_suggested_action (context),
 				 timestamp);
-		result = TRUE;
+		drop_zone = TRUE;
 	}
 
-	return result;
-}
-
-static GtkWidget *
-get_notebook_from_view (GtkWidget *view)
-{
-	GtkWidget *widget;
-
-	widget = view;
-
-	do
-	{
-		widget = gtk_widget_get_parent (widget);
-	}
-	while (!GEDIT_IS_NOTEBOOK (widget));
-
-	return widget;
+	return drop_zone;
 }
 
 static void
@@ -385,38 +356,6 @@ gedit_view_drag_data_received (GtkWidget        *widget,
 			break;
 
 		}
-		case TARGET_TAB:
-		{
-			GtkWidget *notebook;
-			GtkWidget *new_notebook;
-			GtkWidget *page;
-
-			notebook = gtk_drag_get_source_widget (context);
-
-			if (!GTK_IS_WIDGET (notebook))
-			{
-				return;
-			}
-
-			page = *(GtkWidget **) gtk_selection_data_get_data (selection_data);
-			g_return_if_fail (page != NULL);
-
-			/* We need to iterate and get the notebook of the target view
-			   because we can have several notebooks per window */
-			new_notebook = get_notebook_from_view (widget);
-
-			if (notebook != new_notebook)
-			{
-				gedit_notebook_move_tab (GEDIT_NOTEBOOK (notebook),
-							 GEDIT_NOTEBOOK (new_notebook),
-							 GEDIT_TAB (page),
-							 0);
-			}
-
-			gtk_drag_finish (context, TRUE, TRUE, timestamp);
-
-			break;
-		}
 		case TARGET_XDNDDIRECTSAVE:
 		{
 			GeditView *view;
@@ -474,7 +413,7 @@ gedit_view_drag_drop (GtkWidget      *widget,
 		      gint            y,
 		      guint           timestamp)
 {
-	gboolean result;
+	gboolean drop_zone;
 	GdkAtom target;
 	guint info;
 	gboolean found;
@@ -493,28 +432,30 @@ gedit_view_drag_drop (GtkWidget      *widget,
 
 			if (uri != NULL)
 			{
-				g_free (GEDIT_VIEW (widget)->priv->direct_save_uri);
-				GEDIT_VIEW (widget)->priv->direct_save_uri = uri;
+				GeditView *view = GEDIT_VIEW (widget);
+				g_free (view->priv->direct_save_uri);
+				view->priv->direct_save_uri = uri;
 			}
 		}
 
 		gtk_drag_get_data (widget, context, target, timestamp);
-		result = TRUE;
+		drop_zone = TRUE;
 	}
 	else
 	{
 		/* Chain up */
-		result = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_drop (widget,
-										context,
-										x, y,
-										timestamp);
+		drop_zone = GTK_WIDGET_CLASS (gedit_view_parent_class)->drag_drop (widget,
+										   context,
+										   x, y,
+										   timestamp);
 	}
 
-	return result;
+	return drop_zone;
 }
 
-static GtkWidget *
-create_line_numbers_menu (GtkWidget *view)
+static void
+show_line_numbers_menu (GeditView      *view,
+			GdkEventButton *event)
 {
 	GtkWidget *menu;
 	GtkWidget *item;
@@ -525,7 +466,7 @@ create_line_numbers_menu (GtkWidget *view)
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
 					gtk_source_view_get_show_line_numbers (GTK_SOURCE_VIEW (view)));
 
-	g_settings_bind (GEDIT_VIEW (view)->priv->editor_settings,
+	g_settings_bind (view->priv->editor_settings,
 			 GEDIT_SETTINGS_DISPLAY_LINE_NUMBERS,
 			 item,
 			 "active",
@@ -535,17 +476,6 @@ create_line_numbers_menu (GtkWidget *view)
 
 	gtk_widget_show_all (menu);
 
-	return menu;
-}
-
-static void
-show_line_numbers_menu (GtkWidget      *view,
-			GdkEventButton *event)
-{
-	GtkWidget *menu;
-
-	menu = create_line_numbers_menu (view);
-
 	gtk_menu_popup (GTK_MENU (menu),
 			NULL,
 			NULL,
@@ -556,16 +486,17 @@ show_line_numbers_menu (GtkWidget      *view,
 }
 
 static gboolean
-gedit_view_button_press_event (GtkWidget *widget, GdkEventButton *event)
+gedit_view_button_press_event (GtkWidget      *widget,
+			       GdkEventButton *event)
 {
 	if ((event->type == GDK_BUTTON_PRESS) &&
 	    (event->button == GDK_BUTTON_SECONDARY) &&
 	    (event->window == gtk_text_view_get_window (GTK_TEXT_VIEW (widget),
 						        GTK_TEXT_WINDOW_LEFT)))
 	{
-		show_line_numbers_menu (widget, event);
+		show_line_numbers_menu (GEDIT_VIEW (widget), event);
 
-		return TRUE;
+		return GDK_EVENT_STOP;
 	}
 
 	return GTK_WIDGET_CLASS (gedit_view_parent_class)->button_press_event (widget, event);
@@ -600,6 +531,7 @@ gedit_view_realize (GtkWidget *widget)
 	                  "extension-added",
 	                  G_CALLBACK (extension_added),
 	                  view);
+
 	g_signal_connect (view->priv->extensions,
 	                  "extension-removed",
 	                  G_CALLBACK (extension_removed),
@@ -607,7 +539,8 @@ gedit_view_realize (GtkWidget *widget)
 
 	/* We only activate the extensions when the view is realized,
 	 * because most plugins will expect this behaviour, and we won't
-	 * change the buffer later anyway. */
+	 * change the buffer later anyway.
+	 */
 	peas_extension_set_foreach (view->priv->extensions,
 	                            (PeasExtensionSetForeachFunc) extension_added,
 	                            view);
@@ -622,8 +555,9 @@ gedit_view_unrealize (GtkWidget *widget)
 	g_signal_handlers_disconnect_by_func (view->priv->extensions, extension_removed, view);
 
 	/* We need to deactivate the extension on unrealize because it is not
-	   mandatory that a view has been realized when we dispose it, leading
-	   to deactivating the plugin without being activated */
+	 * mandatory that a view has been realized when we dispose it, leading
+	 * to deactivating the plugin without being activated.
+	 */
 	peas_extension_set_foreach (view->priv->extensions,
 	                            (PeasExtensionSetForeachFunc) extension_removed,
 	                            view);
@@ -643,16 +577,16 @@ delete_line (GtkTextView *text_view,
 
 	gtk_text_view_reset_im_context (text_view);
 
-	/* If there is a selection delete the selected lines and
-	 * ignore count */
+	/* If there is a selection delete the selected lines and ignore count. */
 	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
 	{
 		gtk_text_iter_order (&start, &end);
 
 		if (gtk_text_iter_starts_line (&end))
 		{
-			/* Do no delete the line with the cursor if the cursor
-			 * is at the beginning of the line */
+			/* Do not delete the line with the cursor if the cursor
+			 * is at the beginning of the line.
+			 */
 			count = 0;
 		}
 		else
@@ -669,27 +603,36 @@ delete_line (GtkTextView *text_view,
 
 		if (gtk_text_iter_is_end (&end))
 		{
-			if (gtk_text_iter_backward_line (&start) && !gtk_text_iter_ends_line (&start))
+			if (gtk_text_iter_backward_line (&start) &&
+			    !gtk_text_iter_ends_line (&start))
+			{
 				gtk_text_iter_forward_to_line_end (&start);
+			}
 		}
 	}
 	else if (count < 0)
 	{
 		if (!gtk_text_iter_ends_line (&end))
+		{
 			gtk_text_iter_forward_to_line_end (&end);
+		}
 
 		while (count < 0)
 		{
 			if (!gtk_text_iter_backward_line (&start))
+			{
 				break;
+			}
 
-			++count;
+			count++;
 		}
 
 		if (count == 0)
 		{
 			if (!gtk_text_iter_ends_line (&start))
+			{
 				gtk_text_iter_forward_to_line_end (&start);
+			}
 		}
 		else
 		{
@@ -728,14 +671,15 @@ gedit_view_delete_from_cursor (GtkTextView   *text_view,
 			       gint           count)
 {
 	/* We override the standard handler for delete_from_cursor since
-	   the GTK_DELETE_PARAGRAPHS case is not implemented as we like (i.e. it
-	   does not remove the carriage return in the previous line)
+	 * the GTK_DELETE_PARAGRAPHS case is not implemented as we like (i.e. it
+	 * does not remove the carriage return in the previous line).
 	 */
 	switch (type)
 	{
 		case GTK_DELETE_PARAGRAPHS:
 			delete_line (text_view, count);
 			break;
+
 		default:
 			GTK_TEXT_VIEW_CLASS (gedit_view_parent_class)->delete_from_cursor(text_view, type, count);
 			break;
@@ -751,18 +695,15 @@ gedit_view_create_buffer (GtkTextView *text_view)
 static void
 gedit_view_class_init (GeditViewClass *klass)
 {
-	GObjectClass     *object_class = G_OBJECT_CLASS (klass);
-	GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkTextViewClass *text_view_class = GTK_TEXT_VIEW_CLASS (klass);
-	GtkBindingSet    *binding_set;
+	GtkBindingSet *binding_set;
 
 	object_class->dispose = gedit_view_dispose;
 	object_class->constructed = gedit_view_constructed;
 
-	widget_class->focus_out_event = gedit_view_focus_out;
-
-	/*
-	 * Override the gtk_text_view_drag_motion and drag_drop
+	/* Override the gtk_text_view_drag_motion and drag_drop
 	 * functions to get URIs
 	 *
 	 * If the mime type is text/uri-list, then we will accept
@@ -771,13 +712,14 @@ gedit_view_class_init (GeditViewClass *klass)
 	 *
 	 * If the drag context has any other mime type, then pass the
 	 * information onto the GtkTextView's standard handlers.
-	 * (widget_class->function_name).
 	 *
 	 * See bug #89881 for details
 	 */
 	widget_class->drag_motion = gedit_view_drag_motion;
 	widget_class->drag_data_received = gedit_view_drag_data_received;
 	widget_class->drag_drop = gedit_view_drag_drop;
+
+	widget_class->focus_out_event = gedit_view_focus_out;
 	widget_class->button_press_event = gedit_view_button_press_event;
 	widget_class->realize = gedit_view_realize;
 	widget_class->unrealize = gedit_view_unrealize;
@@ -785,17 +727,25 @@ gedit_view_class_init (GeditViewClass *klass)
 	text_view_class->delete_from_cursor = gedit_view_delete_from_cursor;
 	text_view_class->create_buffer = gedit_view_create_buffer;
 
-	/* A new signal DROP_URIS has been added to allow plugins to intercept
-	 * the default dnd behaviour of 'text/uri-list'. GeditView now handles
-	 * dnd in the default handlers of drag_drop, drag_motion and
-	 * drag_data_received. The view emits drop_uris from drag_data_received
-	 * if valid uris have been dropped. Plugins should connect to
-	 * drag_motion, drag_drop and drag_data_received to change this
-	 * default behaviour. They should _NOT_ use this signal because this
-	 * will not prevent gedit from loading the uri
+	/**
+	 * GeditView::drop-uris:
+	 * @view: a #GeditView.
+	 * @uri_list: a %NULL-terminated list of URIs.
+	 *
+	 * The #GeditView::drop-uris signal allows plugins to intercept the
+	 * default drag-and-drop behaviour of 'text/uri-list'. #GeditView
+	 * handles drag-and-drop in the default handlers of
+	 * #GtkWidget::drag-drop, #GtkWidget::drag-motion and
+	 * #GtkWidget::drag-data-received. The view emits the
+	 * #GeditView::drop-uris signal from #GtkWidget::drag-data-received if
+	 * valid URIs have been dropped.  Plugins should connect to
+	 * #GtkWidget::drag-motion, #GtkWidget::drag-drop and
+	 * #GtkWidget::drag-data-received to change this default behaviour. They
+	 * should NOT use this signal because this will not prevent gedit from
+	 * loading the URI.
 	 */
 	view_signals[DROP_URIS] =
-		g_signal_new ("drop_uris",
+		g_signal_new ("drop-uris",
 		              G_TYPE_FROM_CLASS (object_class),
 		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		              G_STRUCT_OFFSET (GeditViewClass, drop_uris),
@@ -817,11 +767,13 @@ gedit_view_class_init (GeditViewClass *klass)
 	                              GDK_CONTROL_MASK,
 	                              "change_case", 1,
 	                              G_TYPE_ENUM, GTK_SOURCE_CHANGE_CASE_UPPER);
+
 	gtk_binding_entry_add_signal (binding_set,
 	                              GDK_KEY_l,
 	                              GDK_CONTROL_MASK,
 	                              "change_case", 1,
 	                              G_TYPE_ENUM, GTK_SOURCE_CHANGE_CASE_LOWER);
+
 	gtk_binding_entry_add_signal (binding_set,
 	                              GDK_KEY_asciitilde,
 	                              GDK_CONTROL_MASK,
@@ -836,8 +788,8 @@ gedit_view_class_init (GeditViewClass *klass)
  * Creates a new #GeditView object displaying the @doc document.
  * @doc cannot be %NULL.
  *
- * Return value: a new #GeditView
- **/
+ * Returns: a new #GeditView.
+ */
 GtkWidget *
 gedit_view_new (GeditDocument *doc)
 {
@@ -857,7 +809,6 @@ gedit_view_cut_clipboard (GeditView *view)
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
 
 	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view),
 	                                      GDK_SELECTION_CLIPBOARD);
@@ -885,7 +836,6 @@ gedit_view_copy_clipboard (GeditView *view)
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
 
 	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view),
 	                                      GDK_SELECTION_CLIPBOARD);
@@ -906,7 +856,6 @@ gedit_view_paste_clipboard (GeditView *view)
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
 
 	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (view),
 	                                      GDK_SELECTION_CLIPBOARD);
@@ -930,7 +879,7 @@ gedit_view_paste_clipboard (GeditView *view)
  *
  * Deletes the text currently selected in the #GtkTextBuffer associated
  * to the view and scroll to the cursor position.
- **/
+ */
 void
 gedit_view_delete_selection (GeditView *view)
 {
@@ -941,7 +890,6 @@ gedit_view_delete_selection (GeditView *view)
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
 
 	gtk_text_buffer_delete_selection (buffer,
 	                                  TRUE,
@@ -959,20 +907,20 @@ gedit_view_delete_selection (GeditView *view)
  * gedit_view_select_all:
  * @view: a #GeditView
  *
- * Selects all the text displayed in the @view.
- **/
+ * Selects all the text.
+ */
 void
 gedit_view_select_all (GeditView *view)
 {
-	GtkTextBuffer *buffer ;
-	GtkTextIter start, end;
+	GtkTextBuffer *buffer;
+	GtkTextIter start;
+	GtkTextIter end;
 
 	gedit_debug (DEBUG_VIEW);
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
 
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
 	gtk_text_buffer_select_range (buffer, &start, &end);
@@ -983,18 +931,17 @@ gedit_view_select_all (GeditView *view)
  * @view: a #GeditView
  *
  * Scrolls the @view to the cursor position.
- **/
+ */
 void
 gedit_view_scroll_to_cursor (GeditView *view)
 {
-	GtkTextBuffer* buffer;
+	GtkTextBuffer *buffer;
 
 	gedit_debug (DEBUG_VIEW);
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
-	g_return_if_fail (buffer != NULL);
 
 	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (view),
 	                              gtk_text_buffer_get_insert (buffer),
@@ -1004,19 +951,18 @@ gedit_view_scroll_to_cursor (GeditView *view)
 	                              0.0);
 }
 
-/* FIXME this is an issue for introspection */
 /**
  * gedit_view_set_font:
  * @view: a #GeditView
- * @def: whether to reset the default font
+ * @default_font: whether to reset to the default font
  * @font_name: the name of the font to use
  *
- * If @def is #TRUE, resets the font of the @view to the default font
- * otherwise sets it to @font_name.
- **/
+ * If @default_font is #TRUE, resets the font of the @view to the default font.
+ * Otherwise sets it to @font_name.
+ */
 void
 gedit_view_set_font (GeditView   *view,
-		     gboolean     def,
+		     gboolean     default_font,
 		     const gchar *font_name)
 {
 	PangoFontDescription *font_desc;
@@ -1025,13 +971,13 @@ gedit_view_set_font (GeditView   *view,
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
-	if (def)
+	if (default_font)
 	{
-		GObject *settings;
+		GeditSettings *settings;
 		gchar *font;
 
 		settings = _gedit_app_get_settings (GEDIT_APP (g_application_get_default ()));
-		font = gedit_settings_get_system_font (GEDIT_SETTINGS (settings));
+		font = gedit_settings_get_system_font (settings);
 
 		font_desc = pango_font_description_from_string (font);
 		g_free (font);
