@@ -16,7 +16,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 import os
-
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gedit, Gtk, Gdk, GObject, Gio, GLib
 import platform
 
