@@ -1,4 +1,3 @@
-/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gedit-spell-checker-dialog.c
  * This file is part of gedit
@@ -19,20 +18,12 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include "gedit-spell-checker-dialog.h"
 #include <string.h>
-
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-
 #include <gedit/gedit-utils.h>
 
-#include "gedit-spell-checker-dialog.h"
-#include "gedit-spell-marshal.h"
-
 struct _GeditSpellCheckerDialog
 {
 	GtkWindow parent_instance;
@@ -152,8 +143,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditSpellCheckerDialogClass, ignore),
-			      NULL, NULL,
-			      gedit_marshal_VOID__STRING,
+			      NULL, NULL, NULL,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_STRING);
@@ -163,8 +153,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditSpellCheckerDialogClass, ignore_all),
-			      NULL, NULL,
-			      gedit_marshal_VOID__STRING,
+			      NULL, NULL, NULL,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_STRING);
@@ -174,8 +163,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditSpellCheckerDialogClass, change),
-			      NULL, NULL,
-			      gedit_marshal_VOID__STRING_STRING,
+			      NULL, NULL, NULL,
 			      G_TYPE_NONE,
 			      2,
 			      G_TYPE_STRING,
@@ -186,8 +174,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditSpellCheckerDialogClass, change_all),
-			      NULL, NULL,
-			      gedit_marshal_VOID__STRING_STRING,
+			      NULL, NULL, NULL,
 			      G_TYPE_NONE,
 			      2,
 			      G_TYPE_STRING,
@@ -198,8 +185,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditSpellCheckerDialogClass, add_word_to_personal),
-			      NULL, NULL,
-			      gedit_marshal_VOID__STRING,
+			      NULL, NULL, NULL,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_STRING);
@@ -209,8 +195,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 			      G_OBJECT_CLASS_TYPE (klass),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (GeditSpellCheckerDialogClass, close),
-			      NULL, NULL,
-			      gedit_marshal_VOID__VOID,
+			      NULL, NULL, NULL,
 			      G_TYPE_NONE,
 			      0);
 }
@@ -382,7 +367,7 @@ gedit_spell_checker_dialog_set_spell_checker (GeditSpellCheckerDialog *dlg, Gedi
 	                             gedit_spell_checker_language_to_string (language));
 
 	if (dlg->misspelled_word != NULL)
-		gedit_spell_checker_dialog_set_misspelled_word (dlg, dlg->misspelled_word, -1);
+		gedit_spell_checker_dialog_set_misspelled_word (dlg, dlg->misspelled_word);
 	else
 		gtk_list_store_clear (GTK_LIST_STORE (dlg->suggestions_list_model));
 
@@ -391,8 +376,7 @@ gedit_spell_checker_dialog_set_spell_checker (GeditSpellCheckerDialog *dlg, Gedi
 
 void
 gedit_spell_checker_dialog_set_misspelled_word (GeditSpellCheckerDialog *dlg,
-						const gchar             *word,
-						gint                     len)
+						const gchar             *word)
 {
 	gchar *tmp;
 	GSList *sug;
@@ -401,7 +385,7 @@ gedit_spell_checker_dialog_set_misspelled_word (GeditSpellCheckerDialog *dlg,
 	g_return_if_fail (word != NULL);
 
 	g_return_if_fail (dlg->spell_checker != NULL);
-	g_return_if_fail (!gedit_spell_checker_check_word (dlg->spell_checker, word, -1));
+	g_return_if_fail (!gedit_spell_checker_check_word (dlg->spell_checker, word));
 
 	g_free (dlg->misspelled_word);
 	dlg->misspelled_word = g_strdup (word);
@@ -411,8 +395,7 @@ gedit_spell_checker_dialog_set_misspelled_word (GeditSpellCheckerDialog *dlg,
 	g_free (tmp);
 
 	sug = gedit_spell_checker_get_suggestions (dlg->spell_checker,
-						   dlg->misspelled_word,
-						   -1);
+						   dlg->misspelled_word);
 
 	update_suggestions_list_model (dlg, sug);
 
@@ -524,15 +507,13 @@ static void
 check_word_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 {
 	const gchar *word;
-	gssize len;
 
 	g_return_if_fail (GEDIT_IS_SPELL_CHECKER_DIALOG (dlg));
 
 	word = gtk_entry_get_text (GTK_ENTRY (dlg->word_entry));
-	len = strlen (word);
-	g_return_if_fail (len > 0);
+	g_return_if_fail (gtk_entry_get_text_length (GTK_ENTRY (dlg->word_entry)) > 0);
 
-	if (gedit_spell_checker_check_word (dlg->spell_checker, word, len))
+	if (gedit_spell_checker_check_word (dlg->spell_checker, word))
 	{
 		GtkListStore *store;
 		GtkTreeIter iter;
@@ -552,9 +533,7 @@ check_word_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *d
 	{
 		GSList *sug;
 
-		sug = gedit_spell_checker_get_suggestions (dlg->spell_checker,
-							   word,
-							   len);
+		sug = gedit_spell_checker_get_suggestions (dlg->spell_checker, word);
 
 		update_suggestions_list_model (dlg, sug);
 
@@ -571,8 +550,7 @@ add_word_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg
 	g_return_if_fail (dlg->misspelled_word != NULL);
 
 	gedit_spell_checker_add_word_to_personal (dlg->spell_checker,
-						  dlg->misspelled_word,
-						  -1);
+						  dlg->misspelled_word);
 
 	word = g_strdup (dlg->misspelled_word);
 
@@ -605,8 +583,7 @@ ignore_all_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *d
 	g_return_if_fail (dlg->misspelled_word != NULL);
 
 	gedit_spell_checker_add_word_to_session (dlg->spell_checker,
-						 dlg->misspelled_word,
-						 -1);
+						 dlg->misspelled_word);
 
 	word = g_strdup (dlg->misspelled_word);
 
@@ -629,8 +606,8 @@ change_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *dlg)
 	g_return_if_fail (*change != '\0');
 
 	gedit_spell_checker_set_correction (dlg->spell_checker,
-					    dlg->misspelled_word, -1,
-					    change, -1);
+					    dlg->misspelled_word,
+					    change);
 
 	word = g_strdup (dlg->misspelled_word);
 
@@ -666,8 +643,8 @@ change_all_button_clicked_handler (GtkButton *button, GeditSpellCheckerDialog *d
 	g_return_if_fail (*change != '\0');
 
 	gedit_spell_checker_set_correction (dlg->spell_checker,
-					    dlg->misspelled_word, -1,
-					    change, -1);
+					    dlg->misspelled_word,
+					    change);
 
 	word = g_strdup (dlg->misspelled_word);
 
