@@ -1,4 +1,3 @@
-/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gedit-spell-language-dialog.c
  * This file is part of gedit
@@ -19,18 +18,13 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include "gedit-spell-language-dialog.h"
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gedit/gedit-utils.h>
 #include <gedit/gedit-app.h>
-#include "gedit-spell-language-dialog.h"
 #include "gedit-spell-checker-language.h"
 
-
 enum
 {
 	COLUMN_LANGUAGE_NAME = 0,
