@@ -2,6 +2,7 @@
  * gedit-spell-plugin.c
  *
  * Copyright (C) 2002-2005 Paolo Maggi
+ * Copyright (C) 2015 Sébastien Wilmet
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,18 +18,9 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "gedit-spell-plugin.h"
-#include "gedit-spell-app-activatable.h"
-#include "gedit-spell-utils.h"
-
-#include <string.h> /* For strlen */
 
 #include <glib/gi18n.h>
-
 #include <gedit/gedit-debug.h>
 #include <gedit/gedit-statusbar.h>
 #include <gedit/gedit-app.h>
@@ -36,10 +28,12 @@
 #include <gedit/gedit-window-activatable.h>
 #include <gtksourceview/gtksource.h>
 
+#include "gedit-automatic-spell-checker.h"
+#include "gedit-spell-app-activatable.h"
 #include "gedit-spell-checker.h"
 #include "gedit-spell-checker-dialog.h"
 #include "gedit-spell-language-dialog.h"
-#include "gedit-automatic-spell-checker.h"
+#include "gedit-spell-utils.h"
 
 #ifdef G_OS_WIN32
 #define GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE "spell-language"
@@ -49,7 +43,9 @@
 #define GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED  "metadata::gedit-spell-enabled"
 #endif
 
-#define GEDIT_AUTOMATIC_SPELL_VIEW "GeditAutomaticSpellView"
+#define SPELL_ENABLED_STR "1"
+
+#define VIEW_DATA_KEY "GeditSpellPlugin-ViewData"
 
 static void gedit_window_activatable_iface_init (GeditWindowActivatableInterface *iface);
 
@@ -75,6 +71,20 @@ struct _CheckRange
 	GtkTextMark *current_mark;
 };
 
+typedef struct _ViewData ViewData;
+
+struct _ViewData
+{
+	GeditSpellPlugin *plugin;
+	GeditView *view;
+	GeditAutomaticSpellChecker *auto_spell;
+
+	/* The doc is also needed, to be sure that the signals are disconnected
+	 * on the same doc.
+	 */
+	GeditDocument *doc;
+};
+
 enum
 {
 	PROP_0,
@@ -94,6 +104,9 @@ static void	set_language_cb			(GSimpleAction *action, GVariant *parameter, gpoin
 static void	auto_spell_activate_cb		(GSimpleAction *action, GVariant *parameter, gpointer data);
 static void	auto_spell_change_state_cb	(GSimpleAction *action, GVariant *state, gpointer data);
 
+static void	on_document_loaded		(GeditDocument *doc, ViewData *data);
+static void	on_document_saved		(GeditDocument *doc, ViewData *data);
+
 static GActionEntry action_entries[] =
 {
 	{ "check-spell", spell_cb },
@@ -104,6 +117,62 @@ static GActionEntry action_entries[] =
 static GQuark spell_checker_id = 0;
 static GQuark check_range_id = 0;
 
+static ViewData *
+view_data_new (GeditSpellPlugin *plugin,
+	       GeditView        *view)
+{
+	ViewData *data;
+
+	data = g_slice_new (ViewData);
+	data->plugin = g_object_ref (plugin);
+	data->view = g_object_ref (view);
+	data->auto_spell = NULL;
+
+	data->doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+	g_object_ref (data->doc);
+
+	g_signal_connect (data->doc,
+			  "loaded",
+			  G_CALLBACK (on_document_loaded),
+			  data);
+
+	g_signal_connect (data->doc,
+			  "saved",
+			  G_CALLBACK (on_document_saved),
+			  data);
+
+	return data;
+}
+
+static void
+view_data_free (ViewData *data)
+{
+	if (data == NULL)
+	{
+		return;
+	}
+
+	if (data->doc != NULL)
+	{
+		g_signal_handlers_disconnect_by_func (data->doc, on_document_loaded, data);
+		g_signal_handlers_disconnect_by_func (data->doc, on_document_saved, data);
+
+		g_object_unref (data->doc);
+	}
+
+	if (data->auto_spell != NULL && data->view != NULL)
+	{
+		gedit_automatic_spell_checker_detach_view (data->auto_spell,
+							   GTK_TEXT_VIEW (data->view));
+	}
+
+	g_clear_object (&data->plugin);
+	g_clear_object (&data->view);
+	g_clear_object (&data->auto_spell);
+
+	g_slice_free (ViewData, data);
+}
+
 static void
 gedit_spell_plugin_init (GeditSpellPlugin *plugin)
 {
@@ -126,9 +195,9 @@ gedit_spell_plugin_dispose (GObject *object)
 
 static void
 gedit_spell_plugin_set_property (GObject      *object,
-                                 guint         prop_id,
-                                 const GValue *value,
-                                 GParamSpec   *pspec)
+				 guint         prop_id,
+				 const GValue *value,
+				 GParamSpec   *pspec)
 {
 	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (object);
 
@@ -146,9 +215,9 @@ gedit_spell_plugin_set_property (GObject      *object,
 
 static void
 gedit_spell_plugin_get_property (GObject    *object,
-                                 guint       prop_id,
-                                 GValue     *value,
-                                 GParamSpec *pspec)
+				 guint       prop_id,
+				 GValue     *value,
+				 GParamSpec *pspec)
 {
 	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (object);
 
@@ -165,9 +234,9 @@ gedit_spell_plugin_get_property (GObject    *object,
 }
 
 static void
-set_spell_language_cb (GeditSpellChecker   *spell,
+set_spell_language_cb (GeditSpellChecker               *checker,
 		       const GeditSpellCheckerLanguage *lang,
-		       GeditDocument 	   *doc)
+		       GeditDocument                   *doc)
 {
 	const gchar *key;
 
@@ -177,12 +246,13 @@ set_spell_language_cb (GeditSpellChecker   *spell,
 	key = gedit_spell_checker_language_to_key (lang);
 	g_return_if_fail (key != NULL);
 
-	gedit_document_set_metadata (doc, GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE,
-				     key, NULL);
+	gedit_document_set_metadata (doc,
+				     GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE, key,
+				     NULL);
 }
 
 static void
-set_language_from_metadata (GeditSpellChecker *spell,
+set_language_from_metadata (GeditSpellChecker *checker,
 			    GeditDocument     *doc)
 {
 	const GeditSpellCheckerLanguage *lang = NULL;
@@ -198,16 +268,16 @@ set_language_from_metadata (GeditSpellChecker *spell,
 
 	if (lang != NULL)
 	{
-		g_signal_handlers_block_by_func (spell, set_spell_language_cb, doc);
-		gedit_spell_checker_set_language (spell, lang);
-		g_signal_handlers_unblock_by_func (spell, set_spell_language_cb, doc);
+		g_signal_handlers_block_by_func (checker, set_spell_language_cb, doc);
+		gedit_spell_checker_set_language (checker, lang);
+		g_signal_handlers_unblock_by_func (checker, set_spell_language_cb, doc);
 	}
 }
 
 static GeditSpellChecker *
 get_spell_checker_from_document (GeditDocument *doc)
 {
-	GeditSpellChecker *spell;
+	GeditSpellChecker *checker;
 	gpointer data;
 
 	gedit_debug (DEBUG_PLUGINS);
@@ -218,16 +288,16 @@ get_spell_checker_from_document (GeditDocument *doc)
 
 	if (data == NULL)
 	{
-		spell = gedit_spell_checker_new ();
+		checker = gedit_spell_checker_new ();
 
-		set_language_from_metadata (spell, doc);
+		set_language_from_metadata (checker, doc);
 
 		g_object_set_qdata_full (G_OBJECT (doc),
 					 spell_checker_id,
-					 spell,
-					 (GDestroyNotify) g_object_unref);
+					 checker,
+					 g_object_unref);
 
-		g_signal_connect (spell,
+		g_signal_connect (checker,
 				  "set_language",
 				  G_CALLBACK (set_spell_language_cb),
 				  doc);
@@ -235,24 +305,20 @@ get_spell_checker_from_document (GeditDocument *doc)
 	else
 	{
 		g_return_val_if_fail (GEDIT_IS_SPELL_CHECKER (data), NULL);
-		spell = GEDIT_SPELL_CHECKER (data);
+		checker = data;
 	}
 
-	return spell;
+	return checker;
 }
 
 static CheckRange *
 get_check_range (GeditDocument *doc)
 {
-	CheckRange *range;
-
 	gedit_debug (DEBUG_PLUGINS);
 
-	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 
-	range = (CheckRange *) g_object_get_qdata (G_OBJECT (doc), check_range_id);
-
-	return range;
+	return g_object_get_qdata (G_OBJECT (doc), check_range_id);
 }
 
 static void
@@ -265,7 +331,7 @@ update_current (GeditDocument *doc,
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	g_return_if_fail (doc != NULL);
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (current >= 0);
 
 	range = get_check_range (doc);
@@ -330,18 +396,24 @@ set_check_range (GeditDocument *doc,
 		range = g_new0 (CheckRange, 1);
 
 		range->start_mark = gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (doc),
-				"check_range_start_mark", &iter, TRUE);
+								 "check_range_start_mark",
+								 &iter,
+								 TRUE);
 
 		range->end_mark = gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (doc),
-				"check_range_end_mark", &iter, FALSE);
+							       "check_range_end_mark",
+							       &iter,
+							       FALSE);
 
 		range->current_mark = gtk_text_buffer_create_mark (GTK_TEXT_BUFFER (doc),
-				"check_range_current_mark", &iter, TRUE);
+								   "check_range_current_mark",
+								   &iter,
+								   TRUE);
 
 		g_object_set_qdata_full (G_OBJECT (doc),
-				 check_range_id,
-				 range,
-				 (GDestroyNotify)g_free);
+					 check_range_id,
+					 range,
+					 g_free);
 	}
 
 	if (gedit_spell_utils_skip_no_spell_check (start, end))
@@ -383,7 +455,9 @@ set_check_range (GeditDocument *doc,
 }
 
 static gchar *
-get_current_word (GeditDocument *doc, gint *start, gint *end)
+get_current_word (GeditDocument *doc,
+		  gint          *start,
+		  gint          *end)
 {
 	const CheckRange *range;
 	GtkTextIter end_iter;
@@ -392,7 +466,7 @@ get_current_word (GeditDocument *doc, gint *start, gint *end)
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	g_return_val_if_fail (doc != NULL, NULL);
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 	g_return_val_if_fail (start != NULL, NULL);
 	g_return_val_if_fail (end != NULL, NULL);
 
@@ -400,12 +474,14 @@ get_current_word (GeditDocument *doc, gint *start, gint *end)
 	g_return_val_if_fail (range != NULL, NULL);
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),
-			&end_iter, range->end_mark);
+					  &end_iter,
+					  range->end_mark);
 
 	range_end = gtk_text_iter_get_offset (&end_iter);
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),
-			&current_iter, range->current_mark);
+					  &current_iter,
+					  range->current_mark);
 
 	end_iter = current_iter;
 
@@ -421,8 +497,10 @@ get_current_word (GeditDocument *doc, gint *start, gint *end)
 
 	gedit_debug_message (DEBUG_PLUGINS, "Current word extends [%d, %d]", *start, *end);
 
-	if (!(*start < *end))
+	if (*start >= *end)
+	{
 		return NULL;
+	}
 
 	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc),
 					  &current_iter,
@@ -468,14 +546,14 @@ goto_next_word (GeditDocument *doc)
 
 static gchar *
 get_next_misspelled_word (GeditView *view,
-                          gint      *word_start_offset,
-                          gint      *word_end_offset)
+			  gint      *word_start_offset,
+			  gint      *word_end_offset)
 {
 	GeditDocument *doc;
 	CheckRange *range;
 	gint start, end;
 	gchar *word;
-	GeditSpellChecker *spell;
+	GeditSpellChecker *checker;
 
 	g_return_val_if_fail (view != NULL, NULL);
 
@@ -485,32 +563,40 @@ get_next_misspelled_word (GeditView *view,
 	range = get_check_range (doc);
 	g_return_val_if_fail (range != NULL, NULL);
 
-	spell = get_spell_checker_from_document (doc);
-	g_return_val_if_fail (spell != NULL, NULL);
+	checker = get_spell_checker_from_document (doc);
+	g_return_val_if_fail (checker != NULL, NULL);
 
 	word = get_current_word (doc, &start, &end);
 	if (word == NULL)
+	{
 		return NULL;
+	}
 
 	gedit_debug_message (DEBUG_PLUGINS, "Word to check: %s", word);
 
-	while (gedit_spell_checker_check_word (spell, word, -1))
+	while (gedit_spell_checker_check_word (checker, word))
 	{
 		g_free (word);
 
 		if (!goto_next_word (doc))
+		{
 			return NULL;
+		}
 
 		/* may return null if we reached the end of the selection */
 		word = get_current_word (doc, &start, &end);
 		if (word == NULL)
+		{
 			return NULL;
+		}
 
 		gedit_debug_message (DEBUG_PLUGINS, "Word to check: %s", word);
 	}
 
 	if (!goto_next_word (doc))
+	{
 		update_current (doc, gtk_text_buffer_get_char_count (GTK_TEXT_BUFFER (doc)));
+	}
 
 	if (word != NULL)
 	{
@@ -532,8 +618,8 @@ get_next_misspelled_word (GeditView *view,
 
 static void
 select_misspelled_word (GeditView *view,
-                        gint       word_start_offset,
-                        gint       word_end_offset)
+			gint       word_start_offset,
+			gint       word_end_offset)
 {
 	GtkTextBuffer *buffer;
 	GtkTextIter word_start_iter;
@@ -573,9 +659,7 @@ ignore_cb (GeditSpellCheckerDialog *dlg,
 
 	select_misspelled_word (view, word_start_offset, word_end_offset);
 
-	gedit_spell_checker_dialog_set_misspelled_word (GEDIT_SPELL_CHECKER_DIALOG (dlg),
-							word,
-							-1);
+	gedit_spell_checker_dialog_set_misspelled_word (GEDIT_SPELL_CHECKER_DIALOG (dlg), word);
 
 	g_free (word);
 }
@@ -605,9 +689,13 @@ change_cb (GeditSpellCheckerDialog *dlg,
 
 	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start, range->mw_start);
 	if (range->mw_end < 0)
+	{
 		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
+	}
 	else
+	{
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end, range->mw_end);
+	}
 
 	w = gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start, &end, TRUE);
 	g_return_if_fail (w != NULL);
@@ -660,9 +748,13 @@ change_all_cb (GeditSpellCheckerDialog *dlg,
 
 	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &start, range->mw_start);
 	if (range->mw_end < 0)
+	{
 		gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc), &end);
+	}
 	else
+	{
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &end, range->mw_end);
+	}
 
 	w = gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &start, &end, TRUE);
 	g_return_if_fail (w != NULL);
@@ -711,7 +803,7 @@ add_word_cb (GeditSpellCheckerDialog *dlg,
 static void
 language_dialog_response (GtkDialog         *dlg,
 			  gint               res_id,
-			  GeditSpellChecker *spell)
+			  GeditSpellChecker *checker)
 {
 	if (res_id == GTK_RESPONSE_OK)
 	{
@@ -719,7 +811,9 @@ language_dialog_response (GtkDialog         *dlg,
 
 		lang = gedit_spell_language_get_selected_language (GEDIT_SPELL_LANGUAGE_DIALOG (dlg));
 		if (lang != NULL)
-			gedit_spell_checker_set_language (spell, lang);
+		{
+			gedit_spell_checker_set_language (checker, lang);
+		}
 	}
 
 	gtk_widget_destroy (GTK_WIDGET (dlg));
@@ -727,13 +821,13 @@ language_dialog_response (GtkDialog         *dlg,
 
 static void
 set_language_cb (GSimpleAction *action,
-                 GVariant      *parameter,
-                 gpointer       data)
+		 GVariant      *parameter,
+		 gpointer       data)
 {
 	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (data);
 	GeditSpellPluginPrivate *priv;
 	GeditDocument *doc;
-	GeditSpellChecker *spell;
+	GeditSpellChecker *checker;
 	const GeditSpellCheckerLanguage *lang;
 	GtkWidget *dlg;
 	GtkWindowGroup *wg;
@@ -746,10 +840,10 @@ set_language_cb (GSimpleAction *action,
 	doc = gedit_window_get_active_document (priv->window);
 	g_return_if_fail (doc != NULL);
 
-	spell = get_spell_checker_from_document (doc);
-	g_return_if_fail (spell != NULL);
+	checker = get_spell_checker_from_document (doc);
+	g_return_if_fail (checker != NULL);
 
-	lang = gedit_spell_checker_get_language (spell);
+	lang = gedit_spell_checker_get_language (checker);
 
 	data_dir = peas_extension_base_get_data_dir (PEAS_EXTENSION_BASE (plugin));
 	dlg = gedit_spell_language_dialog_new (GTK_WINDOW (priv->window),
@@ -766,21 +860,21 @@ set_language_cb (GSimpleAction *action,
 	g_signal_connect (dlg,
 			  "response",
 			  G_CALLBACK (language_dialog_response),
-			  spell);
+			  checker);
 
 	gtk_widget_show (dlg);
 }
 
 static void
 spell_cb (GSimpleAction *action,
-          GVariant      *parameter,
-          gpointer       data)
+	  GVariant      *parameter,
+	  gpointer       data)
 {
 	GeditSpellPlugin *plugin = GEDIT_SPELL_PLUGIN (data);
 	GeditSpellPluginPrivate *priv;
 	GeditView *view;
 	GeditDocument *doc;
-	GeditSpellChecker *spell;
+	GeditSpellChecker *checker;
 	GtkWidget *dlg;
 	GtkTextIter start, end;
 	gint word_start_offset, word_end_offset;
@@ -797,8 +891,8 @@ spell_cb (GSimpleAction *action,
 	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 	g_return_if_fail (doc != NULL);
 
-	spell = get_spell_checker_from_document (doc);
-	g_return_if_fail (spell != NULL);
+	checker = get_spell_checker_from_document (doc);
+	g_return_if_fail (checker != NULL);
 
 	if (gtk_text_buffer_get_char_count (GTK_TEXT_BUFFER (doc)) <= 0)
 	{
@@ -838,7 +932,7 @@ spell_cb (GSimpleAction *action,
 	}
 
 	data_dir = peas_extension_base_get_data_dir (PEAS_EXTENSION_BASE (plugin));
-	dlg = gedit_spell_checker_dialog_new_from_spell_checker (spell, data_dir);
+	dlg = gedit_spell_checker_dialog_new_from_spell_checker (checker, data_dir);
 	g_free (data_dir);
 	gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
 	gtk_window_set_transient_for (GTK_WINDOW (dlg),
@@ -852,46 +946,46 @@ spell_cb (GSimpleAction *action,
 
 	g_signal_connect (dlg, "add_word_to_personal", G_CALLBACK (add_word_cb), view);
 
-	gedit_spell_checker_dialog_set_misspelled_word (GEDIT_SPELL_CHECKER_DIALOG (dlg),
-							word,
-							-1);
+	gedit_spell_checker_dialog_set_misspelled_word (GEDIT_SPELL_CHECKER_DIALOG (dlg), word);
 
 	g_free (word);
 
 	gtk_widget_show (dlg);
 	select_misspelled_word (view, word_start_offset, word_end_offset);
-
 }
 
 static void
-set_auto_spell (GeditWindow   *window,
-                GeditView     *view,
-                gboolean       active)
+set_auto_spell (ViewData *data,
+		gboolean  active)
 {
-	GeditAutomaticSpellChecker *autospell;
-	GeditSpellChecker *spell;
-	GeditDocument *doc;
-
-	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-
-	spell = get_spell_checker_from_document (doc);
-	g_return_if_fail (spell != NULL);
-
-	autospell = gedit_automatic_spell_checker_get_from_document (doc);
-
-	if (active)
+	if (!active)
 	{
-		if (autospell == NULL)
+		if (data->auto_spell != NULL && data->view != NULL)
 		{
-			autospell = gedit_automatic_spell_checker_new (doc, spell);
-			gedit_automatic_spell_checker_attach_view (autospell, view);
-			gedit_automatic_spell_checker_recheck_all (autospell);
+			gedit_automatic_spell_checker_detach_view (data->auto_spell,
+								   GTK_TEXT_VIEW (data->view));
 		}
+
+		g_clear_object (&data->auto_spell);
 	}
-	else
+	else if (data->auto_spell == NULL)
 	{
-		if (autospell != NULL)
-			gedit_automatic_spell_checker_free (autospell);
+		GeditSpellChecker *checker;
+		GtkTextBuffer *buffer;
+
+		checker = get_spell_checker_from_document (data->doc);
+		g_return_if_fail (checker != NULL);
+
+		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data->view));
+		g_return_if_fail (buffer == GTK_TEXT_BUFFER (data->doc));
+
+		data->auto_spell = gedit_automatic_spell_checker_new (GTK_SOURCE_BUFFER (data->doc),
+								      checker);
+
+		gedit_automatic_spell_checker_attach_view (data->auto_spell,
+							   GTK_TEXT_VIEW (data->view));
+
+		gedit_automatic_spell_checker_recheck_all (data->auto_spell);
 	}
 }
 
@@ -933,7 +1027,7 @@ auto_spell_activate_cb (GSimpleAction *action,
 		 */
 		gedit_document_set_metadata (doc,
 					     GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED,
-					     active ? "1" : NULL,
+					     active ? SPELL_ENABLED_STR : NULL,
 					     NULL);
 	}
 }
@@ -957,7 +1051,13 @@ auto_spell_change_state_cb (GSimpleAction *action,
 	view = gedit_window_get_active_view (priv->window);
 	if (view != NULL)
 	{
-		set_auto_spell (priv->window, view, active);
+		ViewData *data = g_object_get_data (G_OBJECT (view), VIEW_DATA_KEY);
+
+		if (data != NULL)
+		{
+			set_auto_spell (data, active);
+		}
+
 		g_simple_action_set_state (action, g_variant_new_boolean (active));
 	}
 }
@@ -997,55 +1097,55 @@ update_ui (GeditSpellPlugin *plugin)
 
 	if (view != NULL)
 	{
-		GeditDocument *doc;
 		GeditTab *tab;
-		GeditTabState state;
-		gboolean autospell;
+		GtkTextBuffer *buffer;
 
-		doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 		tab = gedit_window_get_active_tab (priv->window);
-		state = gedit_tab_get_state (tab);
-		autospell = (doc != NULL &&
-		             gedit_automatic_spell_checker_get_from_document (doc) != NULL);
+		g_return_if_fail (gedit_tab_get_view (tab) == view);
 
 		/* If the document is loading we can't get the metadata so we
-		   endup with an useless speller */
-		if (state == GEDIT_TAB_STATE_NORMAL)
+		 * endup with an useless speller.
+		 */
+		if (gedit_tab_get_state (tab) == GEDIT_TAB_STATE_NORMAL)
 		{
-			g_action_change_state (auto_spell_action, g_variant_new_boolean (autospell));
+			ViewData *data;
+			gboolean auto_spell_enabled;
+
+			data = g_object_get_data (G_OBJECT (view), VIEW_DATA_KEY);
+			auto_spell_enabled = data != NULL && data->auto_spell != NULL;
+
+			g_action_change_state (auto_spell_action,
+					       g_variant_new_boolean (auto_spell_enabled));
 		}
 
+		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+
 		g_simple_action_set_enabled (G_SIMPLE_ACTION (check_spell_action),
-		                             gtk_text_buffer_get_char_count (GTK_TEXT_BUFFER (doc)) > 0);
+		                             gtk_text_buffer_get_char_count (buffer) > 0);
 	}
 }
 
 static void
-set_auto_spell_from_metadata (GeditSpellPlugin *plugin,
-			      GeditView        *view)
+set_auto_spell_from_metadata (ViewData *data)
 {
+	GeditSpellPlugin *plugin = data->plugin;
 	gboolean active = FALSE;
 	gchar *active_str;
-	GeditDocument *doc;
-	GeditDocument *active_doc;
+	GeditView *active_view;
 
-	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
-	active_str = gedit_document_get_metadata (doc,
-						  GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED);
-
-	if (active_str)
+	active_str = gedit_document_get_metadata (data->doc, GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED);
+	if (active_str != NULL)
 	{
-		active = *active_str == '1';
-
+		active = g_str_equal (active_str, SPELL_ENABLED_STR);
 		g_free (active_str);
 	}
 
-	set_auto_spell (plugin->priv->window, view, active);
+	set_auto_spell (data, active);
 
-	/* In case that the doc is the active one we mark the spell action */
-	active_doc = gedit_window_get_active_document (plugin->priv->window);
+	/* In case that the view is the active one we mark the spell action */
+	active_view = gedit_window_get_active_view (plugin->priv->window);
 
-	if (active_doc == doc)
+	if (active_view == data->view)
 	{
 		GAction *action;
 
@@ -1056,39 +1156,35 @@ set_auto_spell_from_metadata (GeditSpellPlugin *plugin,
 }
 
 static void
-on_document_loaded (GeditDocument    *doc,
-		    GeditSpellPlugin *plugin)
+on_document_loaded (GeditDocument *doc,
+		    ViewData      *data)
 {
-	GeditSpellChecker *spell;
-	GeditView *view;
+	GeditSpellChecker *checker;
 
-	spell = GEDIT_SPELL_CHECKER (g_object_get_qdata (G_OBJECT (doc),
-							 spell_checker_id));
-	if (spell != NULL)
+	checker = GEDIT_SPELL_CHECKER (g_object_get_qdata (G_OBJECT (doc),
+							   spell_checker_id));
+	if (checker != NULL)
 	{
-		set_language_from_metadata (spell, doc);
+		set_language_from_metadata (checker, doc);
 	}
 
-	view = GEDIT_VIEW (g_object_get_data (G_OBJECT (doc), GEDIT_AUTOMATIC_SPELL_VIEW));
-
-	set_auto_spell_from_metadata (plugin, view);
+	set_auto_spell_from_metadata (data);
 }
 
 static void
-on_document_saved (GeditDocument    *doc,
-		   GeditSpellPlugin *plugin)
+on_document_saved (GeditDocument *doc,
+		   ViewData      *data)
 {
-	GeditAutomaticSpellChecker *autospell;
-	GeditSpellChecker *spell;
+	GeditSpellChecker *checker;
 	const gchar *key;
 
 	/* Make sure to save the metadata here too */
-	autospell = gedit_automatic_spell_checker_get_from_document (doc);
-	spell = GEDIT_SPELL_CHECKER (g_object_get_qdata (G_OBJECT (doc), spell_checker_id));
 
-	if (spell != NULL)
+	checker = GEDIT_SPELL_CHECKER (g_object_get_qdata (G_OBJECT (doc), spell_checker_id));
+
+	if (checker != NULL)
 	{
-		key = gedit_spell_checker_language_to_key (gedit_spell_checker_get_language (spell));
+		key = gedit_spell_checker_language_to_key (gedit_spell_checker_get_language (checker));
 	}
 	else
 	{
@@ -1097,7 +1193,7 @@ on_document_saved (GeditDocument    *doc,
 
 	gedit_document_set_metadata (doc,
 	                             GEDIT_METADATA_ATTRIBUTE_SPELL_ENABLED,
-	                             autospell != NULL ? "1" : NULL,
+				     data->auto_spell != NULL ? SPELL_ENABLED_STR : NULL,
 	                             GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE,
 	                             key,
 	                             NULL);
@@ -1109,24 +1205,15 @@ tab_added_cb (GeditWindow      *window,
 	      GeditSpellPlugin *plugin)
 {
 	GeditView *view;
-	GeditDocument *doc;
+	ViewData *data;
 
 	view = gedit_tab_get_view (tab);
-	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
-	/* we need to pass the view with the document as there is no way to
-	   attach the view to the automatic spell checker. */
-	g_object_set_data (G_OBJECT (doc), GEDIT_AUTOMATIC_SPELL_VIEW, view);
-
-	g_signal_connect (doc,
-			  "loaded",
-			  G_CALLBACK (on_document_loaded),
-			  plugin);
-
-	g_signal_connect (doc,
-			  "saved",
-			  G_CALLBACK (on_document_saved),
-			  plugin);
+	data = view_data_new (plugin, view);
+	g_object_set_data_full (G_OBJECT (view),
+				VIEW_DATA_KEY,
+				data,
+				(GDestroyNotify) view_data_free);
 }
 
 static void
@@ -1134,13 +1221,8 @@ tab_removed_cb (GeditWindow      *window,
 		GeditTab         *tab,
 		GeditSpellPlugin *plugin)
 {
-	GeditDocument *doc;
-
-	doc = gedit_tab_get_document (tab);
-	g_object_set_data (G_OBJECT (doc), GEDIT_AUTOMATIC_SPELL_VIEW, NULL);
-
-	g_signal_handlers_disconnect_by_func (doc, on_document_loaded, plugin);
-	g_signal_handlers_disconnect_by_func (doc, on_document_saved, plugin);
+	GeditView *view = gedit_tab_get_view (tab);
+	g_object_set_data (G_OBJECT (view), VIEW_DATA_KEY, NULL);
 }
 
 static void
@@ -1169,16 +1251,25 @@ gedit_spell_plugin_activate (GeditWindowActivatable *activatable)
 	for (l = views; l != NULL; l = g_list_next (l))
 	{
 		GeditView *view = GEDIT_VIEW (l->data);
+		ViewData *data = view_data_new (plugin, view);
 
-		set_auto_spell_from_metadata (plugin, view);
+		g_object_set_data_full (G_OBJECT (view),
+					VIEW_DATA_KEY,
+					data,
+					(GDestroyNotify) view_data_free);
+
+		set_auto_spell_from_metadata (data);
 	}
 
-	priv->tab_added_id =
-		g_signal_connect (priv->window, "tab-added",
-				  G_CALLBACK (tab_added_cb), activatable);
-	priv->tab_removed_id =
-		g_signal_connect (priv->window, "tab-removed",
-				  G_CALLBACK (tab_removed_cb), activatable);
+	priv->tab_added_id = g_signal_connect (priv->window,
+					       "tab-added",
+					       G_CALLBACK (tab_added_cb),
+					       activatable);
+
+	priv->tab_removed_id = g_signal_connect (priv->window,
+						 "tab-removed",
+						 G_CALLBACK (tab_removed_cb),
+						 activatable);
 }
 
 static void
@@ -1219,10 +1310,14 @@ gedit_spell_plugin_class_init (GeditSpellPluginClass *klass)
 	object_class->get_property = gedit_spell_plugin_get_property;
 
 	if (spell_checker_id == 0)
+	{
 		spell_checker_id = g_quark_from_string ("GeditSpellCheckerID");
+	}
 
 	if (check_range_id == 0)
+	{
 		check_range_id = g_quark_from_string ("CheckRangeID");
+	}
 
 	g_object_class_override_property (object_class, PROP_WINDOW, "window");
 }
