@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GEDIT_SPELL_UTILS_H__
@@ -27,9 +25,10 @@
 
 G_BEGIN_DECLS
 
-gboolean gedit_spell_utils_is_digit (const char *text, gssize length);
+gboolean	gedit_spell_utils_is_digit		(const gchar *text);
 
-gboolean gedit_spell_utils_skip_no_spell_check (GtkTextIter *start, GtkTextIter *end);
+gboolean	gedit_spell_utils_skip_no_spell_check	(GtkTextIter *start,
+							 GtkTextIter *end);
 
 G_END_DECLS
 
